@@ -237,6 +237,50 @@ struct mail_index_transaction_commit_result {
 	unsigned int ignored_modseq_changes;
 };
 
+struct mail_index_base_optimization_settings {
+	/* Rewrite the index when the number of bytes that needs to be read
+	   from the .log on refresh is between these min/max values. */
+	uoff_t rewrite_min_log_bytes;
+	uoff_t rewrite_max_log_bytes;
+};
+
+struct mail_index_log_optimization_settings {
+	/* Rotate transaction log after it's a) min_size or larger and it was
+	   created at least min_age_secs or b) larger than max_size. */
+	uoff_t min_size;
+	uoff_t max_size;
+	unsigned int min_age_secs;
+
+	/* Delete .log.2 when it's older than log2_stale_secs. Don't be too
+	   eager, because older files are useful for QRESYNC and dsync. */
+	unsigned int log2_max_age_secs;
+};
+
+struct mail_index_cache_optimization_settings {
+	/* Drop fields that haven't been accessed for n seconds */
+	unsigned int unaccessed_field_drop_secs;
+	/* If cache record becomes larger than this, don't add it. */
+	unsigned int record_max_size;
+
+	/* Never compress the file if it's smaller than this */
+	uoff_t compress_min_size;
+	/* Compress the file when n% of records are deleted */
+	unsigned int compress_delete_percentage;
+	/* Compress the file when n% of rows contain continued rows.
+	   For example 200% means that the record has 2 continued rows, i.e.
+	   it exists in 3 separate segments in the cache file. */
+	unsigned int compress_continued_percentage;
+	/* Compress the file when we need to follow more than n next_offsets to
+	   find the latest cache header. */
+	unsigned int compress_header_continue_count;
+};
+
+struct mail_index_optimization_settings {
+	struct mail_index_base_optimization_settings index;
+	struct mail_index_log_optimization_settings log;
+	struct mail_index_cache_optimization_settings cache;
+};
+
 struct mail_index;
 struct mail_index_map;
 struct mail_index_view;
@@ -251,6 +295,10 @@ void mail_index_free(struct mail_index **index);
    can be used to specify which transaction types to fsync. */
 void mail_index_set_fsync_mode(struct mail_index *index, enum fsync_mode mode,
 			       enum mail_index_fsync_mask mask);
+/* Try to set the index's permissions based on its index directory. Returns
+   TRUE if successful (directory existed), FALSE if mail_index_set_permissions()
+   should be called. */
+bool mail_index_use_existing_permissions(struct mail_index *index);
 void mail_index_set_permissions(struct mail_index *index,
 				mode_t mode, gid_t gid, const char *gid_origin);
 /* Set locking method and maximum time to wait for a lock
@@ -258,14 +306,10 @@ void mail_index_set_permissions(struct mail_index *index,
 void mail_index_set_lock_method(struct mail_index *index,
 				enum file_lock_method lock_method,
 				unsigned int max_timeout_secs);
-/* Rotate transaction log after it's a) min_size or larger and it was created
-   at least min_created_ago_secs or b) larger than max_size. Delete .log.2 when
-   it's older than log2_stale_secs. The defaults are min_size=32kB, max_size=1M,
-   min_created_ago_secs=5min, log2_stale_secs=2d. */
-void mail_index_set_log_rotation(struct mail_index *index,
-				 uoff_t min_size, uoff_t max_size,
-				 unsigned int min_created_ago_secs,
-				 unsigned int log2_stale_secs);
+/* Override the default optimization-related settings. Anything set to 0 will
+   use the default. */
+void mail_index_set_optimization_settings(struct mail_index *index,
+	const struct mail_index_optimization_settings *set);
 /* When creating a new index file or reseting an existing one, add the given
    extension header data immediately to it. */
 void mail_index_set_ext_init_data(struct mail_index *index, uint32_t ext_id,
